@@ -2,7 +2,7 @@
  * puzzle.cpp -- Copyright (C) 1998 by M. G"otze
  */
 
-#include <iostream.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
 
@@ -24,8 +24,12 @@
 #include <qdatetime.h>
 #include <qlist.h>
 #include <qpoint.h>
+#include <qcursor.h>
+#include <qfileinfo.h>
 
 #include <kapp.h>
+
+
 #include <klocale.h>
 #include <kglobal.h>
 #include <kstddirs.h>
@@ -38,7 +42,6 @@
 
 #include <stdio.h>
 #include "puzzle.h"
-#include "playername.h"
 #include "score.h"
 #include "cursors.h"
 
@@ -69,9 +72,6 @@ KSlidePuzzle::KSlidePuzzle(QWidget *parent, const char *name,
    scn.fixCoeffs[1]=0.625;
    scn.fixCoeffs[2]=16;
 
-	CPlayerName dlgplayername;
-	dlgplayername.exec();
-	playername=dlgplayername.getPlayerName();
 	score=0;
 	nbmoves=0;
 	imagepath=KGlobal::dirs()->findResourceDir("data","kslide/kslideui.rc");
@@ -88,9 +88,10 @@ KSlidePuzzle::KSlidePuzzle(QWidget *parent, const char *name,
 		else
 			break;
 	}
+  kdDebug()<<QString("Found %1 levels").arg(maxlevel)<<endl;
   srand(time(0));
   puz.prevImage = "";
-  tempFileName = 0;
+  tempFileName = "";
 	level=0;
 	maxmoves=-1;
 	currentmoves=-1;
@@ -100,7 +101,7 @@ KSlidePuzzle::KSlidePuzzle(QWidget *parent, const char *name,
 	currenthints=0;
 	gamelevel=3;
   puz.prevMousePos = QCursor::pos();
-  puz.path = qstrdup(imagepath);
+  puz.path = imagepath.latin1();
   puz.mode = 1;                // default mode is classical game with 4x4 tiles
   puz.size = puz.scrambling = 0;
   setFixedSize(scn.frameSize + 2 * scn.puzOffset, scn.frameSize +
@@ -134,12 +135,12 @@ KSlidePuzzle::~KSlidePuzzle()
 {
 	killTimers();
   // only the first five cursor pointers have been allocated!
-  for (int i = 0; i <= 4; i++)
+/*  for (int i = 0; i <= 4; i++)
     delete puz.cursors[i];
   for (tInd i = 0; i < maxTile(); i++)
     delete puz.tiles[i];
   delete tempFileName;
-  delete puz.frame;
+  delete puz.frame;*/
 //  delete ani.buf;
 //  delete audio;
 }
@@ -158,62 +159,50 @@ void KSlidePuzzle::drawPuzzle()
 
 void KSlidePuzzle::timerEvent(QTimerEvent* e)
 {
-QString display;
-if(e->timerId()==gametimerid)
-{
-	killTimer(gametimerid);
-		level--;
-		KMessageBox::sorry( this, "You have reached max game time\nTry again !", "Sorry");
-		order();
-	return;
-}
-if(e->timerId()==timerlevelid)
-{
-	killTimer( timerlevelid );
-	  emit(sigChangeLevel());
-}
-if(e->timerId()==displaytimerid)
-{
-	gametime.addSecs(30);
-	if((usetimer>0)&&(state==0))
-	{
-		display.setNum(usetimer-gametime.minute());
-		display+=i18n(" mins remaining");
-		statusbar->changeItem(display,2);
-		/*erase( width()-200, height()-35, 200,18);
-		drawText(width()-200, height()-20, display);*/
-	}
-	return;
-}
+  QString display;
+  if(e->timerId()==gametimerid)
+  {
+  	killTimer(gametimerid);
+ 		level--;
+ 		KMessageBox::sorry( this, "You have reached max game time\nTry again !", "Sorry");
+ 		order();
+  	return;
+  }
+  if(e->timerId()==timerlevelid)
+  {
+  //	killTimer( timerlevelid );
+   kdDebug()<<QString("e->timerId()==timerlevelid:emit sigChangeLevel")<<endl;
+  //	emit(sigChangeLevel());
+  }
+  if(e->timerId()==displaytimerid)
+  {
+  	gametime.addSecs(30);
+  	if((usetimer>0)&&(state==0))
+  	{
+  		display.setNum(usetimer-gametime.minute());
+  		display+=i18n(" mins remaining");
+  		statusbar->changeItem(display,3);
+  		/*erase( width()-200, height()-35, 200,18);
+  		drawText(width()-200, height()-20, display);*/
+  	}
+  	return;
+  }
  if (state==stLoadingUncompleting) {
 	  kdDebug()<<"In:LoadingUncompleting\n";
       state=stLoadingUncompleting;
       setState(stLoading, TRUE);
       timerid=startTimer(10);
-/*    tInd x, y;
-    spiral(ani.delta--, x, y);
-    bitBlt(this, ani.x + x, ani.y + y, ani.buf);
-    if (ani.delta == ani.tVal) {
-      puz.pos[puz.missingTile] = noTile;
-      updateMovesTos();
-      state=stLoadingUncompleting;
-      setState(stLoading, TRUE);
-      timerid=startTimer(10);
-    }*/
   }  else if (state==stLoading) {
 	  kdDebug()<<"In:Loading\n";
 	  killTimer(timerid);
-		for(int x=0;x<puz.size*puz.size;x++)
+		for(int x=0;x<(int)(puz.size*puz.size);x++)
 		{
-			kdDebug() << QString("drawing %1\n").arg(x);
 			drawTile(x);
 		}
-		kdDebug()<<"Changing state to complete\n";
       state=stMoving;
 		setState(stInAction,FALSE);
       setCursor(5);
       puz.scrambling=false;
-//      timerid=startTimer(25);
   }  else if ((e->timerId()==timerhintid)&&(isGivingHint())) {
   		if(currenthints==maxhints)
   		{
@@ -228,12 +217,9 @@ if(e->timerId()==displaytimerid)
    	 		statusbar->changeItem(display,2);
    	 	}
   		
-  		QPoint *p=tilelist.last();
-  		tilelist.removeLast();
-  		tInd x=p->x();
-  		tInd y=p->y();
-//  		swap(x, y);
-//  		drawTile(p->x());
+  		QPoint p=tilelist.last();
+  		tilelist.remove(tilelist.last());
+  		tInd y=p.y();
 		moveTile(y);
     if (tilelist.isEmpty()) {
       killTimer(timerid);
@@ -305,7 +291,7 @@ if(e->timerId()==displaytimerid)
       setState(stCompleting, FALSE);
       setState(stComplete, TRUE);
       setCursor(cuNormal);
-		kdDebug()<<"Emit sigPuzzleSolved\n";
+		kdDebug()<<"In timerevent, isCompleting(): Emit sigPuzzleSolved\n";
 		emit(sigPuzzleSolved());
     }
   }
@@ -314,8 +300,6 @@ if(e->timerId()==displaytimerid)
     	display.setNum(currentmoves);
     	display+=i18n(" moves remaining");
     	statusbar->changeItem(display,2);
-    	/*erase( width()-200, height()-35, 200,18);
-    	drawText(width()-200, height()-20, display);*/
   }
 }
 
@@ -339,6 +323,7 @@ void KSlidePuzzle::mouseMoveEvent(QMouseEvent *event)
   if (event) {
     x = event->x() - scn.puzOffset - scn.imgOffset;
     y = event->y() - scn.puzOffset - scn.imgOffset;
+
   } else {
     x = puz.prevMousePos.x() - scn.puzOffset - scn.imgOffset;
     y = puz.prevMousePos.y() - scn.puzOffset - scn.imgOffset;
@@ -362,10 +347,10 @@ void KSlidePuzzle::mouseMoveEvent(QMouseEvent *event)
 
 void KSlidePuzzle::scramble()
 {
-	kdDebug()<<"Scramble\n";
   if (!puz.scrambling)
   {
-  	tilelist.clear();
+    tilelist.clear();
+    emit(enableMenus(true));
     scramble((int)((gamelevel==1 ? 0.25 : gamelevel-1) * (puz.size+1) * (puz.size+1)));
   }
   else
@@ -387,7 +372,7 @@ bool KSlidePuzzle::loadImage(tOpt mode, QString fileName)
 {
   if (!isIdle())
     return FALSE;
-kdDebug ()<<QString("Loading Image %1\n").arg(fileName);
+kdDebug ()<<QString("Loading Image:%1").arg(fileName)<<endl;
   if (!fileName.length()) {
     if (!puz.prevImage.length() || (mode & moLoadDefImage) == moLoadDefImage)
       fileName = getFullPicPath(defImageFileName);
@@ -454,6 +439,7 @@ kdDebug ()<<QString("Loading Image %1\n").arg(fileName);
             paint->drawLine(tileSize(size)-1, 0, tileSize(size)-1,
               tileSize(size));
           if (y > 0 && puz.pos[i-6] != fixTile)                         // top?
+
             paint->drawLine(0, 0, tileSize(size), 0);
           if (y < 5 && puz.pos[i+6] != fixTile)                      // bottom?
             paint->drawLine(0, tileSize(size)-1, tileSize(size),
@@ -500,6 +486,7 @@ unsigned int KSlidePuzzle::movesTo(const tInd index)
       target = index-1;
     if (x < puz.size-1 && puz.pos[index+1] == noTile)      // towards the right
       target = index+1;
+
   }
   return target;
 }
@@ -567,7 +554,6 @@ void KSlidePuzzle::moveTile(const tInd index)
 
 void KSlidePuzzle::scramble(const tInd moves)
 {
-	kdDebug()<<QString("Scramble %1\n").arg(moves);
   if (!moves) {
     if (puz.scrambling)
       sigStoppedScrambling();
@@ -581,15 +567,17 @@ void KSlidePuzzle::scramble(const tInd moves)
   if (!puz.scrambling) {
     puz.scrambling = moves;
     puz.prevScramble = noTile;
+    setState(stOrdering, false);
     sigStartedScrambling();
     if (isComplete()) {
-      setState(stUncompleting, TRUE);
+      setState(stUncompleting, true);
       setCursor(cuWait);
       chooseMissingTile();
       tInd x, y;
       index2Coords(puz.missingTile, x, y);
       ani.x = tileSize() * x + scn.imgOffset;
       ani.y = tileSize() * y + scn.imgOffset;
+
       ani.delta = 15;
       ani.tVal = -1;
       ani.buf = new QPixmap(tileSize()/4, tileSize()/4);
@@ -614,7 +602,7 @@ void KSlidePuzzle::scramble(const tInd moves)
 	if(usetimer>0)
 	{
 		gametimerid=startTimer(usetimer*60000);
-		displaytimerid=startTimer(60000);		
+		displaytimerid=startTimer(60000);
 		gametime.setHMS(0,0,0);
 	}
 }
@@ -622,11 +610,11 @@ void KSlidePuzzle::scramble(const tInd moves)
 
 bool KSlidePuzzle::solveCheck(const bool honor)
 {
-  if (!isIdle()) return FALSE;
+  if (!isIdle()) return false;
   tInd i;
   for (i = 0; i < maxTile(); i++)
     if (puz.pos[i] != fixTile && i != puz.missingTile && puz.pos[i] != i)
-      return FALSE;
+      return false;
   tInd x, y;
   index2Coords(puz.missingTile, x, y);
   ani.x = tileSize() * x + scn.imgOffset;
@@ -642,8 +630,11 @@ bool KSlidePuzzle::solveCheck(const bool honor)
   setCursor(cuWait);
   killTimer(gametimerid);
   if (honor)
+  {
+    kdDebug()<<QString("In solveCheck: emit sigPuzzleSolved")<<endl;
     emit(sigPuzzleSolved());
-  return TRUE;
+  }
+  return true;
 }
 
 void KSlidePuzzle::spiral(const tInd index, tInd &x, tInd &y)
@@ -714,19 +705,20 @@ bool KSlidePuzzle::toggleRandomMissingTile()
   return randomMissingTile();
 }
 
-char* KSlidePuzzle::getFullPicPath(const char *file)
+QString KSlidePuzzle::getFullPicPath(const char *file)
 {
-  delete tempFileName;
-  return tempFileName = qstrdup(QString(puz.path) + "/pics/" + file);
+  tempFileName = (QString(puz.path) + "/pics/" + file);
+  kdDebug()<<QString("getFullPicPath=%1").arg(tempFileName)<<endl;
+  return tempFileName;
 }
 
-char* KSlidePuzzle::getFullSoundPath(const char *file)
+const char* KSlidePuzzle::getFullSoundPath(const char *file)
 {
   delete tempFileName;
-  return tempFileName = qstrdup(QString(puz.path) + "/sounds/" + file);
+  return tempFileName = (QString(puz.path) + "/sounds/" + file).latin1();
 }
 
-char* KSlidePuzzle::getDataFileName(const tOpt file)
+const char* KSlidePuzzle::getDataFileName(const tOpt file)
 {
   switch (file) {
     case fnFrame:
@@ -760,7 +752,7 @@ void KSlidePuzzle::slotSaveGame(){
 	fprintf( stream, "%i ",maxmoves );
 	fprintf( stream, "%i ",score );
 
-    tInd i, j;
+    tInd i;
     for (i = 0; i < puz.size*puz.size; i++) {
 /*      if (puz.pos[i] != fixTile && i != puz.missingTile && puz.pos[i] != i) {
         for (j = lastTile(); puz.pos[j] != i; j--);
@@ -768,13 +760,14 @@ void KSlidePuzzle::slotSaveGame(){
       }*/
 		if(puz.tiles[i] && puz.pos[i] != fixTile /*&& i != puz.missingTile*/ )
 		{
-			kdDebug()<<QString("%1 %2\n").arg(i).arg(puz.pos[i]);
 			fprintf(stream, "%i %i ",puz.pos[i], puz.movesTo[i]);		
 		}
     }
     fprintf(stream, "%i ",tilelist.count());
-    for(i=0;i<tilelist.count();i++)
-    	fprintf(stream, "%i %i", tilelist.at(i)->x(), tilelist.at(i)->y());
+    for(i=0;i<tilelist.count();i++) {
+    	QPoint p=*tilelist.at(i);
+    	fprintf(stream, "%i %i", p.x(), p.y());
+	}
 	fclose(stream);
 }
 
@@ -819,24 +812,24 @@ void KSlidePuzzle::slotLoadGame(){
     for(i=0;i<j;i++)
     {
 		fscanf(stream, "%i %i ",&x, &y);
-		tilelist.append(new QPoint(x,y));
+
+		tilelist.append(QPoint(x,y));
 	}
 
 	fclose(stream);
-kdDebug() << "Load Ended\n";
 
-      state=stLoadingUncompleting;
-      setCursor(cuWait);
-      index2Coords(puz.missingTile, x, y);
-      ani.x = tileSize() * x + scn.imgOffset;
-      ani.y = tileSize() * y + scn.imgOffset;
-      ani.delta = 15;
-      ani.tVal = -1;
-      ani.buf = new QPixmap(tileSize()/4, tileSize()/4);
-      ani.buf->fill(backgroundColor());
-kdDebug() << "Starting timer\n";
-      timerid=startTimer(25);
-      return;
+  state=stLoadingUncompleting;
+  setCursor(cuWait);
+  index2Coords(puz.missingTile, x, y);
+  ani.x = tileSize() * x + scn.imgOffset;
+  ani.y = tileSize() * y + scn.imgOffset;
+  ani.delta = 15;
+  ani.tVal = -1;
+  ani.buf = new QPixmap(tileSize()/4, tileSize()/4);
+  ani.buf->fill(backgroundColor());
+  emit enableMenus(true);
+  timerid=startTimer(25);
+  return;
 }
 
 /** Start to order, giving a hint to the gamer */
@@ -846,6 +839,7 @@ void KSlidePuzzle::slotGiveHint(){
   setState(stGivingHint, TRUE);
 //  setState(stOrdering, TRUE);
   setCursor(cuWait);
+  emit(toggleHint(false));
 //  sigStartedOrdering();
  counterGivingHint_j=lastTile();
   timerhintid=startTimer(1000);
@@ -856,7 +850,9 @@ void KSlidePuzzle::changeLevel()
   if (!isIdle())
     return;
 //  repaint();
-	timerlevelid=startTimer(1000);
+  kdDebug()<<QString("In: changeLevel")<<endl;
+//	timerlevelid=startTimer(1000);
+  slotChangeLevel();
 }
 
 void KSlidePuzzle::slotChangeLevel()
@@ -880,39 +876,45 @@ void KSlidePuzzle::slotChangeLevel()
 	if(level==maxlevel)
 	{
 		KMessageBox::information(this, i18n("You won ! Congratulations !!!"));
-   KConfig *config = kapp->config();
-  config->setGroup("PlayersScores");
-	QStringList names=config->readListEntry("Players");
-	QValueList<int> scores=config->readIntListEntry("Scores");
-	QStringList::Iterator itp=names.begin();
-	QValueList<int>::Iterator its=scores.begin();
-	while(itp!=names.end())
-	{
-		if(*its<score)
-			break;
-		itp++;
-		its++;
-	}	
-	names.insert(itp, playername);
-	scores.insert(its, score);
-	if(names.count()>10)
-	{
-		names.remove(names.at(10));
-		scores.remove(scores.at(10));
+    KConfig *config = kapp->config();
+    config->setGroup("PlayersScores");
+  	QStringList names=config->readListEntry("Players");
+  	QValueList<int> scores=config->readIntListEntry("Scores");
+  	QStringList::Iterator itp=names.begin();
+  	QValueList<int>::Iterator its=scores.begin();
+  	while(itp!=names.end())
+  	{
+  		if(*its<score)
+  			break;
+  		itp++;
+  		its++;
+  	}	
+  	names.insert(itp, playername);
+  	scores.insert(its, score);
+  	if(names.count()>10)
+  	{
+  		names.remove(names.at(10));
+  		scores.remove(scores.at(10));
+  	}
+  	config->writeEntry("Players",names);
+  	config->writeEntry("Scores",scores);
+  	config->sync();
+  	CScore dlg;
+  	dlg.exec();
+  	level=0;
 	}
-	 config->writeEntry("Players",names);
-	 config->writeEntry("Scores",scores);
-	config->sync();
-		CScore dlg;
-		dlg.exec();
-		level=0;
-	}
-
-	currentmoves=maxmoves;
-	kdDebug()<<"Puzzle complete, prepare for next level\n";
-	imagename=imagename.sprintf("pics/puzzle%i.jpg",level);
+//  puz.prevMousePos = QCursor::pos();
+  setCursor(5);
+  emit(enableMenus(false));
+//  tilelist.clear();
+		imagename=imagename.sprintf("pics/puzzle%i.jpg",level);
 	imagename=imagepath+imagename;
 	loadImage(moKeepMode, imagename.data());
+//  for (tInd i = 0; i < puz.size * puz.size; i++)
+//    puz.tiles[i] = 0;
+//  puz.size = puz.scrambling = 0;
+  puz.missingTile=0;
+  killTimer( timerlevelid );
 }
 
 /** Change the image's border */
@@ -976,19 +978,24 @@ void KSlidePuzzle::setMaxHints( int hints ){
 
 void KSlidePuzzle::setMaxGameTime( int mins )
 {
-	usetimer=mins;
-	if((mins>0)&&(state==0))
-	{
-		gametimerid=startTimer(mins*60000);
-		displaytimerid=startTimer(60000);
-		gametime.setHMS(0,0,0);
-	}
+  QString display;
+  usetimer=mins;
+  if((mins>0)&&(state==0))
+  {
+    gametimerid=startTimer(mins*60000);
+    displaytimerid=startTimer(60000);
+    display.setNum(mins);
+    display+=i18n(" mins remaining");
+    statusbar->changeItem(display,3);
+    gametime.setHMS(0,0,0);
+  }
 }
 
 /** No descriptions */
 void KSlidePuzzle::slotStopHint(){
   setState(stGivingHint, false);
   setCursor(5);
+  emit(toggleHint(true));
   killTimer(timerhintid);//=startTimer(25);
 }
 
@@ -1001,3 +1008,32 @@ void KSlidePuzzle::setStatusBar(KStatusBar* sb) {
 void KSlidePuzzle::setGameLevel(int level){
 	gamelevel=level+1;
 }
+
+/** Displays the Hall Of Fame (hiscores)*/
+void KSlidePuzzle::slotViewHallOfFame(){
+  CScore dlg;
+  dlg.exec();
+}
+
+/* Set Player's name */
+void KSlidePuzzle::setPlayername(QString name)
+{
+  playername=name;
+}
+
+//#include "puzzle.moc.cpp"
+/** Set the global border skin */
+void KSlidePuzzle::setFrameImage(QString str){
+  if(QFile::exists(str))
+  {
+    QFileInfo fi(str);
+    scn.frameFileName=fi.fileName();
+    puz.frame = new QPixmap(str);
+  }
+}
+
+/** Set the tiles skin */
+void KSlidePuzzle::setTiledImage(QString str){
+  QFileInfo fi(str);
+  scn.tilesFileName=fi.fileName();
+}
